--create schema sch if it does not exist!!
IF NOT EXISTS(SELECT * FROM sys.schemas WHERE name='SCH')
	EXEC('CREATE SCHEMA SCH')
GO
IF(NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[SCH].[Changes]') AND type in (N'U')))
BEGIN
CREATE TABLE SCH.Changes
(
	changeID numeric(16,3) not null,
	dSuccess datetime,
	dFail datetime,
	failmsg varchar(max)
)
END

set ANSI_NULLS ON
set QUOTED_IDENTIFIER ON
go

IF EXISTS (	SELECT	name 
			FROM	sys.objects 
			WHERE	name = N'ExecuteSchemaChanges' 
			AND		SCHEMA_NAME(schema_id) = 'SCH'
			AND		type = 'P')
	DROP PROCEDURE SCH.ExecuteSchemaChanges

go
CREATE PROCEDURE SCH.ExecuteSchemaChanges
AS
BEGIN 
	SET NOCOUNT ON
	DECLARE @targetID tinyint 
	DECLARE @clientID int 
	DECLARE @sqlCommand nvarchar(max)
	DECLARE @changeID numeric(16,3)
	declare @errmsg varchar(max)
	declare @db_name varchar(100) 
	SELECT @db_name=DB_NAME()
	SET @targetID=1
	SET @clientID=0
	SELECT @clientID = ID FROM main.dbo.ClientIndex WHERE dbname=DB_NAME()
		
	-- get rid of changes which have been disabled but not ran
	-- or those which have failed but are marked as rerunfail!
	DELETE SCH.Changes
		FROM SCH.Changes changes
		INNER JOIN main.SCH.ChangeIndex ci ON changes.changeID=ci.ID AND ci.targetID=@targetID AND (ci.clientID IS NULL OR ci.clientID=@clientID)
		WHERE (ci.Disable=1 AND changes.dsuccess IS NULL) OR (ci.ReRunFail=1 AND changes.dfail IS NOT NULL)
	
	INSERT INTO SCH.Changes
	(changeID)
	SELECT ID
		FROM main.SCH.ChangeIndex ci
			LEFT OUTER JOIN SCH.Changes changes ON ci.ID=changes.changeID
		WHERE ci.targetID=@targetID AND (ci.clientID IS NULL OR ci.clientID=@clientID)
			AND changes.changeID IS NULL
	
	DECLARE sqlCommands CURSOR LOCAL FAST_FORWARD FOR
	SELECT changeID
		FROM SCH.Changes changes
		WHERE changes.dSuccess IS NULL
		ORDER BY changes.changeID
	OPEN sqlCommands
	FETCH NEXT FROM sqlCommands INTO @changeID
	
	WHILE @@FETCH_STATUS=0
	BEGIN
		SELECT @sqlCommand = changeText FROM main.SCH.ChangeIndex WHERE ID=@changeID
		
		begin try
			EXEC (@sqlCommand)
			UPDATE SCH.Changes SET dSuccess = GETUTCDATE(), dFail = NULL WHERE changeID=@changeID
		end try
		begin catch			
			set @errmsg = 'Error in '+ISNULL(ERROR_PROCEDURE(),'SCH.ExecuteSchemaChanges')+' for Schema ID ' + CONVERT(varchar,@changeID) + ' at Line Number: ' + CONVERT(varchar,Error_Line()) + ' and message is ' + ERROR_MESSAGE()
			CLOSE sqlCommands
			DEALLOCATE sqlCommands
			UPDATE SCH.Changes SET dFail = GETUTCDATE(), failmsg = @errmsg WHERE changeID=@changeID
			SET NOCOUNT OFF
			raiserror(@errmsg,16,1)
			RETURN
		END CATCH
		
		FETCH NEXT FROM sqlCommands INTO @changeID
	END
	
	CLOSE sqlCommands
	DEALLOCATE sqlCommands

	SET NOCOUNT OFF
END --End of Procedure

GO

GRANT EXECUTE ON SCH.ExecuteSchemaChanges to public
go
